import socket
import json
from struct import pack, unpack

class MarketEngine(object):
    def __init__(self, sym_cid_map):
        self.sym_cid_map = sym_cid_map
        self.cid_sym_map = {v: k for k, v in self.sym_cid_map.items()}
        self.host = socket.gethostname()
        self.ip_port = None
        self.mcast_grp = None
        self.mcast_port = None
        self.symbols = []
        self.exchanges = set()
        return

    def set_control_server(self, ip_port):
        self.ip_port = ip_port

    def set_multicast_cfg(self, mcast_grp, mcast_port):
        self.mcast_grp = mcast_grp
        self.mcast_port = mcast_port

    def add_listen_symbol(self, symbol, exchange, book_level):
        cid = self.cid_sym_map[symbol+'.'+exchange]
        self.symbols.append({'cid': cid, 'port': [symbol, exchange], 'book_level': book_level})
        self.exchanges.add(exchange)

    def send_msg(self, msg, socket):
        message_len = len(msg)
        message_format = '>i' + str(message_len) + 's'
        send_message = pack(message_format, message_len,msg.encode('utf-8'))
        socket.send(send_message)

    def run(self):
        self.server_socket = socket.socket()
        self.server_socket.connect((self.host, self.ip_port))
        # request new instance and wait for pin and instance port
        msg = {"type": "new_market_instance", 
                "cfg": {
                    "instance": {"license_id": "TRAIL001", "license_key": "apifiny123456", "log_path": "/data/cc/trader_service", "log_level": 1, "name": "trader_service", "book_depth": 5}, 
                    "servers": {"redis_server": "127.0.0.1"}, 
                    "multibroad": {"network_card_name": "PC1", "local_port": self.ip_port, "multicast_ip": self.mcast_grp, "multicast_port": self.mcast_port}, 
                    "exchanges": [{"exchange": exch} for exch in self.exchanges], 
                    "symbols": self.symbols}
                    }
        # print(msg)
        msg = json.dumps(msg)
        self.send_msg(msg, self.server_socket)
        data = self.server_socket.recv(1024)
        # print(data)
        length = unpack('i', data[:4])[0]
        resp = json.loads(data[4:length+4].decode('utf-8'))
        print(resp)
        return

if __name__ == "__main__":
    sym_cid_map = {
        1001: "BTCUSDT.BINANCE",
        1002: "ETHUSDT.BINANCE",
        1003: "DOGEUSDT.BINANCE",
        1004: "BTCUSDTSWAP.BINANCE_SWAP"
        }
    mkt_engine = MarketEngine(sym_cid_map)
    mkt_engine.set_control_server(6000)
    mkt_engine.set_multicast_cfg('239.0.0.4', 4141)
    # mkt_engine.add_listen_symbol('BTCUSDT', 'BINANCE', 5)
    # mkt_engine.add_listen_symbol('ETHUSDT', 'BINANCE', 5)
    mkt_engine.add_listen_symbol('DOGEUSDT', 'BINANCE', 5)
    mkt_engine.add_listen_symbol('BTCUSDTSWAP', 'BINANCE_SWAP', 5)
    mkt_engine.run()
