import socket
import logging
import datetime
import time
from struct import pack, unpack

from aats.manager import Manager
from aats.base.symbolinfo import SymbolInfo
# from strategy_examples.simple_taker_strategy import SimpleTakerStrategy
# from strategy_examples.simple_maker_strategy import SimpleMakerStrategy
# from client_socket import ClientSocket

logger = logging.getLogger('main')
logger_name = 'algo_trading_' + str(datetime.datetime.today().strftime("%Y%m%d")) + '.log'
output_file_handler = logging.FileHandler(logger_name)
formatter = logging.Formatter('%(asctime)s- %(funcName)s - [%(levelname)s]: %(message)s')
output_file_handler.setFormatter(formatter)
# stdout_handler = logging.StreamHandler(sys.stdout)
logger.addHandler(output_file_handler)
# logger.addHandler(stdout_handler)
logger.setLevel(logging.DEBUG)

class TradeEngine(object):
    def __init__(self, sym_cid_map):
        start = time.time_ns()
        self.sym_cid_map = sym_cid_map
        self.cid_sym_map = {v: k for k, v in self.sym_cid_map.items()}
        self.si = dict()
        self.cfg = dict()
        self.cfg['exchanges'] = []
        self.cfg['apikeys'] = dict()
        self.cfg['fees'] = dict()
        self.cfg['symbol_info'] = dict()
        self.cfg['md_symbols'] = []
        self.cfg['trade_symbols'] = []
    
    def set_control_server(self, server_port):
        self.cfg['server_port'] = server_port
    
    def set_md_multicast_cfg(self, mcast_grp, mcast_port):
        self.cfg['mcast_grp'] = mcast_grp
        self.cfg['mcast_port'] = mcast_port

    def config_exchange(self, exchange, trade_type):
        self.exchange = exchange
        self.trade_type = trade_type
        self.cfg['exchanges'].append({'exchange': self.exchange, 'trader_type': self.trade_type})

    def set_apikey(self, exchange, key, secret, password, subaccount):
        self.exchange = exchange
        self.key = key
        self.secret = secret
        self.password = password
        self.subaccount = subaccount
        self.cfg['apikeys'][exchange] = {'key': self.key, 'secret': self.secret, 'pass': self.password, 'subaccount': self.subaccount}

    def set_fee(self, exchange, taker_fee, maker_fee):
        self.taker_fee = taker_fee
        self.maker_fee = maker_fee
        self.cfg['fees'][exchange] = {"take": self.taker_fee, "make": self.maker_fee}

    def add_md_symbol(self, symbol, exchange):
        cid = self.cid_sym_map[symbol+'.'+exchange]
        self.cfg['md_symbols'].append({'cid': cid, 'port': [symbol, exchange]})
        self.si[cid] = SymbolInfo(cid, symbol, exchange)
    
    def add_trade_symbol(self, symbol, exchange):
        self.trade_symbol = symbol
        self.trade_exchange = exchange
        self.trade_cid = self.cid_sym_map[symbol+'.'+exchange]
        self.cfg['trade_symbols'].append({'cid': self.trade_cid, 'port': [symbol, exchange]})
        self.si[self.trade_cid] = SymbolInfo(self.trade_cid, symbol, exchange)
        # self.si[cid].trade_flag = True

    def add_strategy(self, strategy):
        self.strategy = strategy


    def run(self):
        self.manager = Manager()
        self.manager.subscrib_strategy(self.strategy)

        self.manager.init_manager(self.cfg)
        self.strategy.init_strategy(self.si, self.trade_cid, self.trade_symbol, self.trade_exchange)
        self.manager.start()
        # self.manager.show_all_trade_instances()

    def stop(self):
        print("close strategy and close open orders")
        self.strategy.close()
        print("close thread")
        self.manager.stop()
        time.sleep(1)
        print("closed")



# if __name__ == "__main__":

#     sym_cid_map = {
#         1001: "BTCUSDT.BINANCE",
#         1002: "ETHUSDT.BINANCE"
#         }


#     trade_engine = TradeEngine(sym_cid_map)

#     # setup public config
#     trade_engine.set_md_multicast_cfg(mcast_grp='239.0.0.3', mcast_port=4141)
#     trade_engine.add_md_symbol(symbol='BTCUSDT', exchange='BINANCE')
#     # trade_engine.add_md_symbol(symbol='DOGEUSDT', exchange='BINANCE')

#     # setup private config
#     trade_engine.set_control_server(server_port=6000)
#     trade_engine.config_exchange(exchange='BINANCE', trade_type='sandbox')
#     trade_engine.set_apikey(exchange="BINANCE", 
#                             key="02SvhZEYG1p92JWdekP75XQKayqfLxmjHWNEfWU1KrCPjJ5xrLcOU1YHZ5SUBVFA", 
#                             secret="iKnZvDKMGQuEINhjDX8gbIVJDLl48fV6GFLL5gcFT8Sfj9yxGrnP7uFm7AAVWeFP", 
#                             password="",
#                             subaccount="")
#     trade_engine.set_fee('BINANCE', 0.0003, 0)
#     trade_engine.add_trade_symbol(symbol='BTCUSDT', exchange='BINANCE')

#     # setup your strategy
#     # my_strategy = SimpleMakerStrategy()
#     my_strategy = SimpleTakerStrategy(msecs=10000)
#     trade_engine.add_strategy(my_strategy)

#     # start run
#     try:
#         trade_engine.run()
#     except KeyboardInterrupt as err:
#         # trade_engine.stop()
#         print("close strategy and close open orders")
#         # trade_engine.manager.stop()
#         my_strategy.close()
#         time.sleep(1)
#         print("trade closed")

        

